require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_runbookdirectory',
  dscmeta_resource_friendly_name: 'RunbookDirectory',
  dscmeta_resource_name: 'MSFT_xRunbookDirectory',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xSCSMA',
  dscmeta_module_version: '2.1.0.0',
  docs: 'The DSC RunbookDirectory resource type.
         Automatically generated from version 2.1.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_ensure: {
      type: "Enum['Published', 'Draft', 'Absent']",
      desc: 'The import state of runbooks found at RunbookPath. This can be Published, Draft, or Absent',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_port: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Port of the SMA web site. Defaults to the SMA default of 9090.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_matches: {
      type: 'Optional[Boolean]',
      desc: 'Describes the validity of the imported Runbook(s).',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_runbookpath: {
      type: 'String',
      desc: 'Path to Runbook(s) to be imported. Accepts wildcards.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_webserviceendpoint: {
      type: 'String',
      desc: "URL of SMA's web service endpoint.",
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
