require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xscsmarunbookworkerserversetup',
  dscmeta_resource_friendly_name: 'xSCSMARunbookWorkerServerSetup',
  dscmeta_resource_name: 'MSFT_xSCSMARunbookWorkerServerSetup',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xSCSMA',
  dscmeta_module_version: '2.1.0.0',
  docs: 'The DSC xSCSMARunbookWorkerServerSetup resource type.
         Automatically generated from version 2.1.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_sendtelemetryreports: {
      type: 'Optional[String]',
      desc: 'Usage and Diagnostics Data sent to Microsoft.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_setupcredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credential to be used to perform the installation.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_sourcepath: {
      type: 'String',
      desc: 'UNC path to the root of the source files for installation.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sqlinstance: {
      type: 'String',
      desc: 'Name of the SQL Instance for the SMA database.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_msupdate: {
      type: 'Optional[String]',
      desc: 'Use Microsoft Update.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_logmsiinstall: {
      type: 'Optional[String]',
      desc: 'Set to activate logging during install of MSI.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sourcefolder: {
      type: 'Optional[String]',
      desc: 'Folder within the source path containing the source files for installation.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Enum['Present', 'Absent']",
      desc: 'An enumerated value that describes if the SMA Runbook Worker server is expected to be installed on the machine. Present {default}   Absent    ',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_msilogname: {
      type: 'Optional[String]',
      desc: 'Logfile name for MSI install.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_service: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Service account of the web service application pool.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_msilogpath: {
      type: 'Optional[String]',
      desc: 'Path to log file for MSI install.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_etwmanifest: {
      type: 'Optional[String]',
      desc: 'Log to ETW.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_productkey: {
      type: 'Optional[String]',
      desc: 'Product key for licensed installations.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_installfolder: {
      type: 'Optional[String]',
      desc: 'Installation folder for SMA.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sqlserver: {
      type: 'String',
      desc: 'Name of the SQL Server for the SMA database.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_serviceusername: {
      type: 'Optional[String]',
      desc: 'Output username of the Service account of the web service application pool.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sqldatabase: {
      type: 'Optional[String]',
      desc: 'Name of the SMA database.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
